# fylr

[![release](https://github.com/programmfabrik/fylr/actions/workflows/release.yml/badge.svg?branch=production)](https://github.com/programmfabrik/fylr/actions/workflows/release.yml)
[![unit-and-apitests](https://github.com/programmfabrik/fylr/actions/workflows/tests.yml/badge.svg)](https://github.com/programmfabrik/fylr/actions/workflows/tests.yml)

## Getting started

> :warning: This setup is geard towards **Debian** based distros.
> Other *nix users will find this guide useful, however they will need to use their OS's packaging facility to install dependencies.

### Basic setup

Update your package index cache

```bash
apt-get update
```

1. Golang

**Go 1.20** is required as the minimum supported version for the project. Follow the [standard Golang instructions](https://golang.org/doc/install). The end result is, you should be able to run the following commands without issues

```bash
go version
go mod
```

2. Nodejs

**Nodejs 14.15.4. LTS** is _recommended_ as the minimum supported version for the project. The suggested setup is through
[NVM, Nodejs Version Manager](https://github.com/nvm-sh/nvm)

<details>
    <summary>3. [Optional] Python 2</summary>
<p>

**Python2** is required as the currently compatible version for integrating with the [EasyDB Webfrontend](https://github.com/programmfabrik/easydb-webfrontend). Follow the link for further installation instructions

To install it run

```bash
apt-get install python2
```

</p>
</details>

### Project dependencies

To compile fylr, the following dependencies are required to be on your system and their executable files accessible to your user:

* ffmpeg
* imagemagick
* libopenjp
* rsync
* gcc/g++
* make
* curl
* pkg-config
* exiftool
* ghostscript
* libreoffice-core
* xsltproc
* coffeescript
* sass

1. Apt-get the following:

```bash
apt-get install -y ffmpeg imagemagick libopenjp2-7 libopenjp2-7-dev \
    rsync gcc make curl pkg-config \
    exiftool ghostscript xsltproc

apt-get install -y --no-install-recommends libreoffice-core
```

2. Npm the following:

```bash
npm i -g coffeescript@1.12.7 sass
```

### Clone project

```bash
git clone git@github.com:programmfabrik/fylr.git
cd fylr/
git submodule update --init --recursive
```

### Build project

If all dependencies are in place, building is as easy as

```bash
make
```

This will generate binaries in their respective directories under `cmd`.

### Start server

#### Server configuration

Before running the `fylr` binary, we need to prepare a config file

Start from [cmd/fylr/fylr.example.yml](cmd/fylr/fylr.example.yml), please note the following:

1. Regexs from `execserver.*.commands.regex` might require updating depending on the versions of dependencies you ended up installing
2. `apitest.store.config_email` will require your attention

Start by making a copy of [cmd/fylr/fylr.example.yml](cmd/fylr/fylr.example.yml) and keep
it in the same directory as the example file

```bash
cp cmd/fylr/fylr.example.yml cmd/fylr/fylr.yml
```

_The name of the file is significant, though it can be changed by passing `-c` option to the `fylr` binary._

#### Server dependencies

Operational dependencies are managed through [docker-compose.yml](docker-compose.yml).

1. **Required Once** For first time setup, make sure you're in the project's root directory and run

```bash
mkdir -p _data/{docker,binary}/{elastic,postgres,minio}
```

<details>
<summary>Troubleshooting Common Issues</summary>
<p>

1. If elasticsearch image refuses to start citing access denied to its internal data directory, the problem is in the host's (your machine) data volume permissions. Quickest solution is to chmod 777 the `_data` directory

</p>
</details>
&nbsp;

2. `docker-compose up`

<details>
<summary>Troubleshooting Common Issues</summary>
<p>

1. If you've cloned your repoistory on a windows file system while running from WSL, `docker-compose` will most likely fail booting PostgresSQL for not being able to set permissions on PGDATA. Unfortunately there's no known fix for that, so please switch to a linux filesystem.

</p>
</details>
&nbsp;

#### Run

```bash
make server
```

### Test server

Export this environment variable, it's best to store it in your shell profile.

```bash
export APITEST_PARAMS="--server http://root:admin@localhost:8080/api/v1 --stop-on-fail"
```

Then run either

```bash
make demo-doc 
```

OR

```bash
make demo-simple 
```

At this point, you can try running `apitest` and brew yourself some good coffee.

```bash
make apitest
```

<details>
<summary>Troubleshooting Common Issues</summary>
<p>

1. If you receive this error

```log
ERRO[0000] [ 0] error sending request: Could not do http request: Get "/session": unsupported protocol scheme ""
```

Then you forgot to set the environment variable `APITEST_PARAMS` mentioned in the step before this.

</p>
</details>
&nbsp;


### [OPTIONAL] Access the web frontend

If you followed the optional step in [Getting Started](#getting-started) then you should
have cloned the [Web frontend Repo](https://github.com/programmfabrik/easydb-webfrontend)

Make sure you specify the location of the repo in `fylr.yml` configuration file, by setting `server.webfrontend`.

Follow the instructions in Web frontend repo, then restart fylr server again by exiting the process and running `make server` once more.

If you used the default URL configurations then you should be able to load [http://localhost:8080](http://localhost:8080)

Default login credentials are:
- **Username**: *root*
- **Password**: *admin*

This is a successful login

![""](docs/successful_login.png)

&nbsp;
&nbsp;

---

## Docker Container Setup

You can check the docker setup in the [docker directory](docker/). This should also be your current working directory so for this type of setup. `cd docker`

### Build The Image

To build the image, all you have to do is make sure you're inside [docker directory](docker/) and then run

```bash
make container
```
<details>
<summary>[Optional] Custom docker build arguments </summary>
<p>

```bash
export DOCKER_CONTAINER_BUILD_ARGUMENTS='--label "git_commit_sha=${{ github.sha }}" --label "maintainer=Leon Steinhäuser <leon.steinhaeuser@fylr.io>"'
```

</p>
</details>
&nbsp;

### Run Using Docker Compose

```bash
# If you haven't created this _data directory, please do so now
mkdir -p _data/{docker,binary}/{elastic,postgres,minio,sqlite}
make fylr-start
```

There's also `make fylr-stop` to stop fylr.

## Releases and Commits

* Detailed information about *releases* can be found on the page [Releases](https://github.com/programmfabrik/fylr/wiki/Releases).
* Detailed information about the *git commit* syntax can be found on page [Commits](https://github.com/programmfabrik/fylr/wiki/Commits).
