class AdminEventManagerEvent extends SystemListManagerItem

	initOpts: ->
		super()
		@addOpts
			event:
				check: "PlainObject"
	readOpts: ->
		super()
		@__event = @_event

	renderContent: ->
		@addColumn(new CUI.ListViewColumn(text: @getTimestamp()))
		@addColumn(new CUI.ListViewColumn(text: @getUserGeneratedDisplayname()))
		@addColumn(new CUI.ListViewColumn(text: AdminEventManager.locaType(@getType())))
		@addColumn(new CUI.ListViewColumn(text: @getType()))
		# @addColumn(new CUI.ListViewColumn(text: @getEvent().object_id))
		new CUI.Label(text: @getID())

	isDeletable: ->
		true

	getEvent: ->
		@__event.event

	getType: ->
		@getEvent().type

	getID: ->
		@getEvent()._id

	getUserGeneratedDisplayname: ->
		if @__event.user
			# The server does not return the generated displayname. As fallback we return the ID.
			# If we want to show the displayname we will have to fetch it somehow.
			@__event.user._generated_displayname or "#{@__event.user._id}"
		else
			$$("admin.event.manager.user_generated_displayname.system")

	registerDataChangedListener: ->

	load: ->
		# This is a condition to avoid fetching the event details in multiple selection.
		if @manager.getTree().getSelectedRows()?.length > 1
			return CUI.resolvedPromise()
		ez5.api.event
			api: "/"+@getID()
		.done (@__event) =>

	getTimestamp: ->
		ez5.format_date_and_time(@getEvent().timestamp, true)

	getPaneContent: ->
		ev = @getEvent()
		fields = []

		getAssetId = =>
			value = ev.info?.asset_id
			if not value and ev.object_id and ev.basetype == "asset"
				value = ev.object_id
			return value

		for k in [
			"type"
			"_id"
			"timestamp"
			"remote_addr"
			"object_version"
			"schema"
			"user_generated_displayname"
			"basetype"
			"objecttype"
			"object_id"
			"global_object_id"
			"system_object_id"
			"asset_id"
			"info"
		]

			switch k
				when "timestamp"
					value = @getTimestamp()
				when "user_generated_displayname"
					value = @getUserGeneratedDisplayname()
				when "type"
					value = AdminEventManager.locaType(ev[k])+' ['+ev[k]+']'
				when "global_object_id"
					value = ev.global_object_id or ev.info?.global_object_id
				when "system_object_id"
					value = ev.info?.system_object_id
				when "object_id"
					assetId = getAssetId()
					if not assetId # assetId defined means that object_id is id asset it is skipped.
						value = ev.object_id
				when "asset_id"
					value = getAssetId()
				else
					value = ev[k]

			if CUI.util.isEmpty(value)
				continue

			if CUI.isPlainObject(value) || CUI.isArray(value)
				objectDumper = new CUI.ObjectDumper(object: value, parse_json: true)
				fields.push(@__getOutputContentField(objectDumper, k))
			else if value instanceof CUI.DOMElement
				fields.push(@__getOutputContentField(value, k))
			else
				showObjectModalButton = null

				field =
					form:
						label: $$("admin.event.manager.column.#{k}|text")

				showObjectModalButton = @__getObjectModalButton(value, k, ev)
				if showObjectModalButton
					label = new CUI.Label(text: value)

					field.type = CUI.OutputContent
					field.content = new CUI.HorizontalList(content: [label, CUI.dom.space(), showObjectModalButton])
				else
					field.type = CUI.Output
					field.text = value+""

				fields.push(field)

		new CUI.Form
			fields: fields
		.start()

	getEditorPane: ->
		new CUI.SimplePane
			class: "cui-pane--window"
			header_left: @manager.getTitle()
			padded: true
			content: @getPaneContent()
			footer_right: =>
				new LocaButton
					loca_key: "admin.event.manager.event.button.close"
					onClick: =>
						@deselect()

	__getOutputContentField: (content, key) ->
		type: CUI.OutputContent
		maximize_horizontal: true
		content: content
		form:
			label: $$("admin.event.manager.column.#{key}|text")

	__getObjectModalButton: (id, type, event) ->

		if event.type == "OBJECT_DELETE"
			return

		if type == "object_id" and not event.objecttype
			return

		if type not in ["global_object_id", "system_object_id", "asset_id", "object_id"]
			return

		return new CUI.Button(
			size: "mini"
			text: $$("admin.event.manager.column.show_object.button")
			onClick: =>
				ez5.splash.show("admin.event.manager.splash.loading-object")

				if type == "global_object_id"
					loadPromise = ResultObject.loadFromGlobalObjectId(id)
				else if type == "system_object_id"
					loadPromise = (new ResultObject()).loadFromSystemObjectId(id, "long")
				else if type == "object_id"
					resultObject = new ResultObject()
					search_data =
						format: "long"
						search: [
							type: "in"
							fields: [event.objecttype + "._id"]
							in: [id]
						]
					loadPromise = resultObject.loadFromSearch(search_data)
				else if type == "asset_id"
					objectTypeManager = new ObjecttypeManager().addObjecttypes((table) => not table.getSchema()._instance)
					easFields = objectTypeManager.getEASFields("expert").map((easField) =>
						return easField.fullName() + "._id"
					)

					resultObject = new ResultObject()
					search_data =
						format: "long"
						search: [
							type: "in"
							fields: easFields
							in: [id]
						]
					loadPromise = resultObject.loadFromSearch(search_data)

				loadPromise.done((resultObject) =>
					resultObject.showModal()
				).always( =>
					ez5.splash.hide()
				).fail( =>
					CUI.alert(text: $$("admin.event.manager.alert.object_not_found"))
				)
		)
		return

	delete: ->
		@manager.__deleteMultipleById([@getID()])