import sys
import traceback
import time
import json
import os.path
import requests
import os.path
import hashlib
import datetime

from requests.auth import HTTPBasicAuth
from requests_oauthlib import OAuth1
from urllib.parse import urljoin, parse_qs

from context import GenericServerError, EasydbException, UserError

def dump(obj):
    global logger;
    logger.debug(json.dumps(obj, indent=4))

def easydb_server_start(easydb_context):
    global logger;
    logger = easydb_context.get_logger('transport.wordpress')
    logger.debug('initialize Wordpress plugin')

    easydb_context.register_callback('export_transport', {
        'transport_type': 'wordpress',
        'callback': 'transport_wp',
    })

    easydb_context.register_callback('api', {
        'name': 'oauth1',
        'callback': 'api_oauth1',
    })


def api_oauth1(easydb_context, parameters):
    try:
        return _api_oauth1(easydb_context, parameters)
    except Exception as e:
        traceback.print_exc()
        raise

def _api_oauth1(easydb_context, parameters):
    global logger;
    logger = easydb_context.get_logger('transport.wordpress')
    config = easydb_context.get_config()
    plugin = easydb_context.get_plugin("easydb-wordpress-plugin")
    external_url = config.get('system').get('server').get('external_url')
    # oauth_callback = external_url+plugin.get('base_url').replace('/static', '')+'oauth1'
    oauth_callback = external_url+plugin.get('plugin_url')+'oauth1'

    dump(plugin)
    dump(parameters)

    def user_error(code, parameters):
        return {
            'headers': {'Content-Type': 'text/html'},
            'body': """<html><head><script>
window.opener.postMessage("""+json.dumps(
                {"error": {
                        "realm": "user",
                        "code": code,
                        "parameters": parameters}
                 })+""",'*');
</script></head></html>""",
            }

    # this is the callback from WP, we parse this and call ourselves once
    # again for the final step
    if parameters['query_string_parameters'].get('oauth_verifier'):
        oauth_verifier = parameters['query_string_parameters'].get('oauth_verifier')[0]
        oauth_token = parameters['query_string_parameters'].get('oauth_token')[0]

        dump(oauth_token)
        dump(oauth_verifier)

        return {
            'headers': {
                'Content-Type': 'text/html'
                },
            'body': """<html><head><script>
var wp_oauth = JSON.parse(window.sessionStorage.getItem("wp_oauth"));
wp_oauth.verifier = '"""+oauth_verifier+"""';
var params = [];
for (k in wp_oauth) {
  params.push(encodeURIComponent(k)+'='+encodeURIComponent(wp_oauth[k]));
}
url = '"""+oauth_callback+"""?'+params.join('&');
document.location.assign(url);
</script></head></html>
            """,
            }


    if 'wp_url' not in parameters['query_string_parameters']:
        return user_error('wordpress.oauth_error', {'error': "Parameter wp_url missing."})

    wp_url = parameters['query_string_parameters'].get('wp_url')[0]
    r = requests.get(wp_url+'/wp-json')

    try:
        authentication = json.loads(r.content).get('authentication')
        wp_urls = authentication.get('oauth1')
    except:
        return user_error('wordpress.oauth_error', {'error': "Authentication 'oauth1' not allowed."})

    dump(wp_urls)

    # this is the callback from ourselves after we got the callback from WP
    if parameters['query_string_parameters'].get('verifier'):

        wp_oauth = {}
        for key in [
            'wp_url',
            'client_key',
            'client_secret',
            'resource_owner_key',
            'resource_owner_secret',
            'verifier'
            ]:
            wp_oauth[key] = parameters['query_string_parameters'][key][0]

        dump(wp_oauth)

        oauth = OAuth1(wp_oauth['client_key'],
                       client_secret=wp_oauth['client_secret'],
                       resource_owner_key=wp_oauth['resource_owner_key'],
                       resource_owner_secret=wp_oauth['resource_owner_secret'],
                       verifier=wp_oauth['verifier'])

        r = requests.post(url=wp_urls['access'], auth=oauth)

        credentials = parse_qs(r.content.decode('utf-8'))

        dump(credentials)

        resource_owner_key = credentials.get('oauth_token')[0]
        resource_owner_secret = credentials.get('oauth_token_secret')[0]

        msg = json.dumps({
                "oauth_token": credentials.get('oauth_token')[0],
                "oauth_token_secret": credentials.get('oauth_token_secret')[0]
                })

        return {
            'headers': {'Content-Type': 'text/html'},
            'body': """<html><head><script>
window.opener.postMessage("""+msg+""",'*');
</script></head></html>""",
            }


    # initial request coming from easydb
    if parameters['query_string_parameters'].get('client_key'):

        client_key = parameters['query_string_parameters'].get('client_key')[0]
        client_secret = parameters['query_string_parameters'].get('client_secret')[0]

        dump(client_key)
        dump(client_secret)
        dump(oauth_callback)

        oauth = OAuth1(client_key, client_secret=client_secret)
        r = requests.post(url=wp_urls['request'], params={'oauth_callback': oauth_callback}, auth=oauth)

        if r.status_code != 200:
            return user_error('wordpress.oauth_error', {'error': r.content.decode('utf-8', 'replace')})

        credentials = parse_qs(r.content.decode('utf-8'))
        resource_owner_key = credentials.get('oauth_token')[0]
        resource_owner_secret = credentials.get('oauth_token_secret')[0]

        wp_oauth = json.dumps(
            {
            'wp_url': wp_url,
            'client_key': client_key,
            'client_secret': client_secret,
            'resource_owner_key': resource_owner_key,
            'resource_owner_secret': resource_owner_secret
            }
            )

        dump(wp_oauth)

        authorize_url = wp_urls.get('authorize') + '?oauth_token=' + resource_owner_key
        # redirect browser to wordpress
        # wordpress will redirect back to us

        return {
            'headers': {
                'Content-Type': 'text/html'
                },
            'body': """<html><head><script>
window.sessionStorage.setItem("wp_oauth", '"""+wp_oauth+"""');
document.location.assign('"""+authorize_url+"""');
</script></head></html>
            """
            }

    return user_error('wordpress.oauth_error', {'error': "Required parameter missing"})


def transport_wp(easydb_context, protocol = None):
    global logger;
    logger = easydb_context.get_logger('transport.wordpress')
    logger.debug("Transport Wordpress Start.")

    def raise_error(err):
        protocol.add_error("Wordpress: %s" % err)
        raise Exception(err)

    exp = easydb_context.get_exporter()
    transport = exp.getCurrentTransport()

    if not transport:
        return raise_error("failed to get current transport")

    opts = transport.get('options', {})
    wp_inst_uuid = opts.get("wp_inst_uuid")
    user_generated_displayname = opts.get("user_generated_displayname")

    config = easydb_context.get_config()

    try:
        wp_conf = config["base"]["system"]["wordpress"]
    except:
        return raise_error("wordpress config not found")

    wp_inst = None
    auth = None

    dump(wp_conf)

    for _wp_inst in wp_conf["wp_instances"]:
        auth = json.loads(_wp_inst["auth"])

        if auth["uuid"] == wp_inst_uuid:
            wp_inst = _wp_inst
            break

    if wp_inst is None:
        return raise_error("wordpress instance not found")

    logger.debug("=== WP INST:")
    logger.debug(json.dumps(wp_inst, indent = 4))
    logger.debug("=== WP INST AUTH:")
    logger.debug(json.dumps(auth, indent = 4))
    logger.debug("=== EXPORTER PATH:")
    fpath = exp.getFilesPath()
    logger.debug(json.dumps(fpath))

    protocol.add_notice("Wordpress: %s" % wp_inst.get('url'))

    wp_url = urljoin(wp_inst['url'], '/wp-json/wp/v2/media')

    if auth['auth_type'] == "http-auth":
        req_auth = HTTPBasicAuth(auth['http_auth_login'], auth['http_auth_password'])
    elif auth['auth_type'] == "oauth-1":
        req_auth = OAuth1(
            auth['oauth_client_key'],
            auth['oauth_client_secret'],
            auth['oauth_token'],
            auth['oauth_token_secret']
            )
    else:
        req_auth = None

    logger.debug("=== WP URL: %s" % wp_url)

    if exp.isScheduled():
        event_name = 'ASSET_EXPORT_TRANSPORT_COPY_SCHEDULED'
    else:
        event_name = 'ASSET_EXPORT_TRANSPORT_COPY'

    for fo in exp.getFiles():
        path = fpath + "/" + fo['path']
        filename = os.path.basename(path)

        if fo.get('eas_fileclass') not in ['image']:
            protocol.add_notice("SKIPPED: '"+filename+"', wrong fileclass")
            continue

        headers = {
            "Content-Type": "application/octet-stream",
            "Content-Disposition": 'attachment; filename="'+filename+'"'
            }

        title = filename
        description = 'Imported from easydb: %s. User: %s.' % (datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'), user_generated_displayname)

        # produce a unique id for the file we are pushing

        with open(path, 'rb') as f:
            slug = hashlib.md5(f.read()).hexdigest()

        logger.debug("File: %s %s %s %s" % (path, title, slug, description))

        # let's see if the file exists in WP already

        r = requests.get(
            wp_url,
            params={
                'slug': slug,
                'title': title,
                'description': description
                },
            auth=req_auth
            )

        response = json.loads(r.content)

        if not (200 <= r.status_code <= 299):
            return raise_error(response.get('message').encode('utf-8'))

        files_by_slug = json.loads(r.content)

        if len(files_by_slug) == 0:
            # file is new, push it

            logger.debug("URL: %s SLUG: %s" % (wp_url, slug))

            with open(path, "rb") as f:
                r = requests.post(
                    wp_url,
                    data=f,
                    params={
                        'slug': slug,
                        'title': title,
                        'description': description
                        },
                    headers=headers,
                    auth=req_auth
                    )

                dump("Response Status: %s" % (str(r.status_code)))

                if not (200 <= r.status_code <= 299):
                    return raise_error(r.content)

                log_response = r.content.decode('utf-8', 'replace')
                response = json.loads(r.content)

                dump(response['guid']['raw'])
                protocol.add_notice(filename+" pushed to Wordpress.")

                exp.logEvent({
                        'name': event_name,
                        'pollable': False,
                        'base_type': 'asset',
                        'object_id': fo.get('eas_id'),
                        'event_info': {
                            'wordpress': r.content, # response,
                            'version': fo.get('eas_version'),
                            'class': fo.get('eas_fileclass'),
                            'system_object_id': fo.get('system_object_id'),
                            }
                        })
        else:
            # update file
            # print json.dumps(files_by_slug, indent=4)

            # update the first file only
            update_file = files_by_slug[0]

            r = requests.post(
                wp_url+"/"+str(update_file['id']),
                params={
                    'title': title,
                    'description': description
                    },
                headers=headers,
                auth=req_auth
                )

            log_response = r.content.decode('utf-8', 'replace')
            response = json.loads(r.content)
            dump("Response Status: %s" % (str(r.status_code)))

            if not (200 <= r.status_code <= 299):
                return raise_error(response.get('message').encode('utf-8'))

            dump(response['guid']['raw'])
            protocol.add_notice(filename+' updated in Wordpress. ID:'+str(update_file['id']))

        logger.debug("Transport Wordpress Done.")


        if log_response:
            exp.logEvent({
                'name': 'WORDPRESS_SYNC',
                'pollable': False,
                'base_type': 'asset',
                'object_id': fo.get('eas_id'),
                'event_info': {
                        'wordpress': log_response,
                        'version': fo.get('eas_version'),
                        'class': fo.get('eas_fileclass'),
                        'system_object_id': fo.get('system_object_id'),
                        }
                })
